function [z,A,f0,h,Ayd]=freevib(y,Fs,stype);
%
% Function FREEVIB (Free Vibration Modal Analysis) 
% It determines the instantaneous  modal   parameters 
% and force characteristics of a symmetric 
% linear and non-linear vibration SDOF system. 
% Input:
% Vector y is a vibration signal in time domain,
% stype is a signal type, e.g. displacement, velocity, or acceleration.
% Fs is the sampling frequency [Hz]
% 
% Output:
% z - displacement, A - envelope, f0 - natural frequency [Hz], 
% h - damping coefficient [1/sec], Ayd - envelope of the velocity
% 
% y'' + 2*h*y' +(2*pi*f0)^2*y = 0
%
% EXAMPLE:
% [z, A, f0, h, Ayd]=freevib(y,1000,'d');
%
% LIMITATIONS: 
%    The sampling frequency Fs has to be in the range Fs=(10-100)*f0.
%    The minimum of points in time domain is 3*230+1. 
%     
%  2011 Michael Feldman   
% For use with the book "HILBERT TRANSFORM APPLICATION 
% IN MECHANICAL VIBRATION", John Wiley & Sons, 2011
%

N=230; % The filters length
if length(y)<=3*230+1, error('The length of the signal y must be more than three times the filter order'),end;
if nargin<3, error('Not enough input arguments'), end;
s=strmatch(lower(stype),{'displacement','velocity','acceleration'});

y=y(:); t=(0:(length(y)-1))'/Fs;  

if s==0,error('Wrong signal type');

elseif s==1,
    yH  = hilbfir(y);       % Displacement Hilbert transform
    yd  = diffir(y,Fs);     % Velocity
    ydd = diffir(yd,Fs);    % Aceleration
    yHd = hilbfir(yd);      % Hilbert velocity
    yHdd= hilbfir(ydd);     % Hilbert aceleration
    
elseif s==2
    yd  = y;                % Velocity
    y   = integ(yd,Fs);     % Displacement
    yH  = hilbfir(y);       % Displacement Hilbert transform
    ydd = diffir(yd,Fs);    % Aceleration
    yHd = hilbfir(yd);      % Hilbert velocity
    yHdd= hilbfir(ydd);     % Hilbert aceleration
    
elseif s==3
    ydd=y;                  % Acceleration
    yd  = integ(ydd,Fs);    % Velocity
    y   = integ(yd,Fs);     % Displacement
    yH  = hilbfir(y);       % Displacement Hilbert transform
    yHd = hilbfir(yd);      % Hilbert velocity
    yHdd= hilbfir(ydd);     % Hilbert aceleration

end

% Algebraic transforms and instantaneous modal parameters calculation.
A2=(yH.^2+y.^2); A=sqrt(A2);          % A -- Vibration amplitude,   
var2=(y.*yHd-yd.*yH); 
om0_2=(yHdd.*yd-yHd.*ydd)./(var2+eps);% om_02 -- Undamped natural frequency, transient [rad]                                        
h=0.5*(yH.*ydd-yHdd.*y)./(var2);      % h -- Damping coefficient
Ayd2=(yHd.^2+yd.^2); 
Ayd=sqrt(Ayd2);                       % Ayd -- Velocity amplitude, 

% Low pass filtration (Result  Smoothing)
fp=0.02;

z=  y;                                % Displacement
A   = lpf(A,fp);                      % Amplitude, Displaycement 
f0 = sqrt(abs( lpf(om0_2,fp) ))/2/pi; % Natural frequency [Hz]
h   = lpf(h,fp);                      % Damping  [1/s]
Ayd = lpf(Ayd,fp);                    % Amplitude of Velocity

